/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package ca.mcscert.jtet.expression;

import java.util.Collection;

/**
 *
 * @author matthew
 */
public final class ExpressionBinaryOperation implements Expression, ExpressionWithSubExpression {

    public ExpressionBinaryOperation(Expression lhs, BinaryOperation op, Expression rhs) {
        m_lhs = lhs;
        m_rhs = rhs;
        m_op = op;
    }

    public void setLHS(Expression expr) {
        m_lhs = expr;
    }

    public void setRHS(Expression expr) {
        m_rhs = expr;
    }

    private VariableType getInputTypeForOutput(VariableTypeMarker requestedType) {
        Collection<VariableTypeMarker> allowedInputTypes = m_op.getInputTypeForOutput(requestedType);
        for(VariableTypeMarker allowedType : allowedInputTypes) {
            VariableType inputType = m_lhs.actualOutputTypeForRequestedOutput(allowedType);
            if(inputType != null &&
                    m_rhs.actualOutputTypeForRequestedOutput(inputType) != null) {
                return inputType;
            }
            inputType = m_rhs.actualOutputTypeForRequestedOutput(allowedType);
            if(inputType != null &&
                    m_lhs.actualOutputTypeForRequestedOutput(inputType) != null) {
                return inputType;
            }
        }
        return null;
    }

    @Override
    public VariableType actualOutputTypeForRequestedOutput(VariableTypeMarker requestedType) {
        VariableType inputType = getInputTypeForOutput(requestedType);
        if (inputType != null) {
            return m_op.getOutputForInput(inputType);
        }
        return null;
    }

    @Override
    public String getCheckerOutput(CheckerGenerator generator, VariableType requestedType) {
        VariableType inputType = getInputTypeForOutput(requestedType);
        if (inputType == null) {
            throw new IllegalArgumentException("Requested output type is not valid for the expression's output type (" + requestedType.getClass().getSimpleName() + "!");
        }

        String lhsExp = m_lhs.getCheckerOutput(generator, inputType);
        String rhsExp = m_rhs.getCheckerOutput(generator, inputType);

        return generator.GenerateBinaryOperation(m_op, lhsExp, rhsExp, inputType);
    }

    @Override
    public Expression getSubExpression() {
        return m_rhs;
    }

    @Override
    public void setSubExpression(Expression subExpression) {
        m_rhs = subExpression;
    }

    @Override
    public String toString() {
        return "(" + m_lhs.toString() + " " + m_op.toString() + " " + m_rhs.toString() + ")";
    }

    private Expression m_lhs, m_rhs;
    private BinaryOperation m_op;
}
