/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package ca.mcscert.jtet.parsers;

import ca.mcscert.jtet.expression.BinaryOperation;
import ca.mcscert.jtet.expression.Expression;
import ca.mcscert.jtet.expression.ExpressionUnaryOperation;
import ca.mcscert.jtet.expression.ExpressionValue;
import ca.mcscert.jtet.expression.ExpressionBinaryOperation;
import ca.mcscert.jtet.expression.ExpressionWithSubExpression;
import ca.mcscert.jtet.expression.Literal;
import ca.mcscert.jtet.expression.UnaryOperation;
import ca.mcscert.jtet.expression.Variable;
import ca.mcscert.jtet.expression.VariableCollection;
import ca.mcscert.jtet.parsers.antlr.MatlabParserParser.G1Context;
import ca.mcscert.jtet.parsers.antlr.MatlabParserParser.G2Context;
import ca.mcscert.jtet.parsers.antlr.MatlabParserParser.G3Context;
import ca.mcscert.jtet.parsers.antlr.MatlabParserParser.G4Context;
import ca.mcscert.jtet.parsers.antlr.MatlabParserParser.E0Context;
import ca.mcscert.jtet.parsers.antlr.MatlabParserParser.E1Context;
import ca.mcscert.jtet.parsers.antlr.MatlabParserParser.E2Context;
import ca.mcscert.jtet.parsers.antlr.MatlabParserParser.E3Context;
import ca.mcscert.jtet.parsers.antlr.MatlabParserParser.E4Context;
import ca.mcscert.jtet.parsers.antlr.MatlabParserParser.E5Context;
import ca.mcscert.jtet.parsers.antlr.MatlabParserParser.E6Context;
import ca.mcscert.jtet.parsers.antlr.MatlabParserParser.E7Context;
import ca.mcscert.jtet.parsers.antlr.MatlabParserParser.E8Context;
import ca.mcscert.jtet.parsers.antlr.MatlabParserParser.E9Context;
import ca.mcscert.jtet.parsers.antlr.MatlabParserParser.E10Context;
import ca.mcscert.jtet.parsers.antlr.MatlabParserParser.E11Context;
import ca.mcscert.jtet.parsers.antlr.MatlabParserParser.Literal_numberContext;
import ca.mcscert.jtet.parsers.antlr.MatlabParserParser.Id_plus_indexersContext;
import ca.mcscert.jtet.parsers.antlr.MatlabParserParser.Prefix_operatorContext;
import java.util.ArrayDeque;
import java.util.Deque;
import java.util.Map;

import ca.mcscert.jtet.parsers.antlr.MatlabParserBaseListener;
import ca.mcscert.jtet.parsers.antlr.MatlabParserLexer;
import ca.mcscert.jtet.parsers.antlr.MatlabParserParser;
import org.antlr.v4.runtime.ANTLRInputStream;
import org.antlr.v4.runtime.CommonTokenStream;
import org.antlr.v4.runtime.tree.ParseTree;
import org.antlr.v4.runtime.tree.ParseTreeWalker;

/**
 *
 * @author matthew
 */
final public class MatlabParser {

    private MatlabParser() {
    }

    public static Expression parseMatlabCode(VariableCollection variableListing, String matlabCode) {
        // create a CharStream that reads from standard input
        ANTLRInputStream input = new ANTLRInputStream(matlabCode + "\n");

        // create a lexer that feeds off of input CharStream
        MatlabParserLexer lexer = new MatlabParserLexer(input);

        // create a buffer of tokens pulled from the lexer
        CommonTokenStream tokens = new CommonTokenStream(lexer);

        // create a parser that feeds off the tokens buffer
        MatlabParserParser parser = new MatlabParserParser(tokens);

        ParseTree tree = parser.expression();
        ParseTreeWalker walker = new ParseTreeWalker();
        MatlabParserToExpressions listener = new MatlabParserToExpressions(variableListing.getVariablesAndEnumeratedValues());
        walker.walk(listener, tree);

        return listener.m_rootExpression;
    }

    private static class MatlabParserToExpressions extends MatlabParserBaseListener implements ExpressionWithSubExpression {

        MatlabParserToExpressions(Map<String, Variable> variableListing) {
            m_variableParser = variableListing;
            m_opStack.addFirst(new ExpressionStackContainer(0, this));
        }

        private void incrementLevel() {
            this.m_level++;
        }

        private void decrementLevel() {
            this.m_level--;
            if (!m_hiddenOpStack.isEmpty() && m_hiddenOpStack.peekFirst().level == m_level) {
                // We have a hidden op here.  The normal methods won't find it, so make use of it!
                HiddenOpStackContainer hiddenOp = m_hiddenOpStack.peekFirst();
                addExpressionBinaryOperation(new ExpressionBinaryOperation(null, hiddenOp.op, null));
                if (--hiddenOp.count == 0) {
                    m_hiddenOpStack.removeFirst();
                }
            }

            if (!m_opStack.isEmpty() && m_opStack.peekFirst().level > m_level) {
                m_opStack.removeFirst();
            }
        }

        private void enqueHiddenOp(BinaryOperation op, int count) {
            m_hiddenOpStack.addFirst(new HiddenOpStackContainer(m_level, op, count));
        }

        private void addExpressionValue(ExpressionValue expressionValue) {
            m_opStack.peekFirst().op.setSubExpression(expressionValue);
        }

        private void addExpressionBinaryOperation(ExpressionBinaryOperation op) {
            if (m_rootExpression == null) {
                throw new IllegalStateException("It should not be possible to have"
                        + "an op where the left hand side is a null expression!");
            } else {
                if (m_opStack.peekFirst().level >= m_level) { //This current op happens after the root expression op.  Pull it up.
                    op.setLHS((Expression) m_opStack.removeFirst().op);
                    m_opStack.peekFirst().op.setSubExpression(op);
                } else { //This op happens before.  Push it down the tree.
                    op.setLHS(m_opStack.peekFirst().op.getSubExpression());
                    m_opStack.peekFirst().op.setSubExpression(op);
                }
                m_opStack.addFirst(new ExpressionStackContainer(m_level, op));
            }
        }

        private void addPrefixUnaryOperation(ExpressionUnaryOperation op) {
        /* Unlike Binary operations, unary operations just always insert
         * themselves into the operation stack exactly where they appear.
         * The whole left hand side handling never happens, since there is no
         * left hand side.  So just put it in place.
         *
         * Also, an unary operation is allowed to deal with a null root expression,
         * since it doesn't use it!
         */
            m_opStack.peekFirst().op.setSubExpression(op);
            m_opStack.addFirst(new ExpressionStackContainer(m_level, op));
        }

        private static BinaryOperation getBinaryOpForSymbol(String text) {
            if (text.equals("+")) {
                return BinaryOperation.Addition;
            } else if (text.equals("-")) {
                return BinaryOperation.Minus;
            } else if (text.equals("*")) {
                return BinaryOperation.Multiplication;
            } else if (text.equals("/")) {
                return BinaryOperation.Division;
            } else if (text.equals("^")) {
                return BinaryOperation.Exponentiation;
            } else if (text.equals(">")) {
                return BinaryOperation.GreaterThen;
            } else if (text.equals(">=")) {
                return BinaryOperation.GreaterThenEqual;
            } else if (text.equals("<")) {
                return BinaryOperation.LessThen;
            } else if (text.equals("<=")) {
                return BinaryOperation.LessThenEqual;
            } else if (text.equals("==")) {
                return BinaryOperation.Equals;
            } else if (text.equals("~=")) {
                return BinaryOperation.NotEquals;
            } else if (text.equals("&&")) {
                return BinaryOperation.BooleanAnd;
            } else if (text.equals("||")) {
                return BinaryOperation.BooleanOr;
            } else {
                throw new IllegalArgumentException("No such binary matlab operation defined (" + text + ")!");
            }
        }

        private static UnaryOperation getUnaryOpForSymbol(String text) {
            if (text.equals("~")) {
                return UnaryOperation.Negation;
            } else if (text.equals("-")) {
                return UnaryOperation.Minus;
            } else {
                throw new IllegalArgumentException("No such unary matlab operation defined (" + text + ")!");
            }
        }

        @Override
        public Expression getSubExpression() {
            return m_rootExpression;
        }

        @Override
        public void setSubExpression(Expression subExpression) {
            m_rootExpression = subExpression;
        }


        @Override
        public void enterE0(E0Context ctx) {
            incrementLevel();
        }

        @Override
        public void enterE1(E1Context ctx) {
            incrementLevel();
            if (ctx.LOG_OR().size() > 0) {
                enqueHiddenOp(BinaryOperation.BooleanOr, ctx.LOG_OR().size());
            }
        }

        @Override
        public void enterE2(E2Context ctx) {
            incrementLevel();
            if (ctx.LOG_AND().size() > 0) {
                enqueHiddenOp(BinaryOperation.BooleanAnd, ctx.LOG_AND().size());
            }
        }

        @Override
        public void enterE3(E3Context ctx) {
            incrementLevel();
        }

        @Override
        public void enterE4(E4Context ctx) {
            incrementLevel();
        }

        @Override
        public void enterE5(E5Context ctx) {
            incrementLevel();
        }

        @Override
        public void enterE6(E6Context ctx) {
            incrementLevel();
        }

        @Override
        public void enterE7(E7Context ctx) {
            incrementLevel();
        }

        @Override
        public void enterE8(E8Context ctx) {
            incrementLevel();
        }

        @Override
        public void enterE9(E9Context ctx) {
            incrementLevel();
        }

        @Override
        public void enterE10(E10Context ctx) {
            incrementLevel();
        }

        @Override
        public void enterE11(E11Context ctx) {
            incrementLevel();
        }

        @Override
        public void exitE0(E0Context ctx) {
            decrementLevel();
        }

        @Override
        public void exitE1(E1Context ctx) {
            decrementLevel();
        }

        @Override
        public void exitE2(E2Context ctx) {
            decrementLevel();
        }

        @Override
        public void exitE3(E3Context ctx) {
            decrementLevel();
        }

        @Override
        public void exitE4(E4Context ctx) {
            decrementLevel();
        }

        @Override
        public void exitE5(E5Context ctx) {
            decrementLevel();
        }

        @Override
        public void exitE6(E6Context ctx) {
            decrementLevel();
        }

        @Override
        public void exitE7(E7Context ctx) {
            decrementLevel();
        }

        @Override
        public void exitE8(E8Context ctx) {
            decrementLevel();
        }

        @Override
        public void exitE9(E9Context ctx) {
            decrementLevel();
        }

        @Override
        public void exitE10(E10Context ctx) {
            decrementLevel();
        }

        @Override
        public void exitE11(E11Context ctx) {
            decrementLevel();
        }

        @Override
        public void enterG1(G1Context ctx) {
            addExpressionBinaryOperation(new ExpressionBinaryOperation(null, getBinaryOpForSymbol(ctx.getText()), null));
        }

        @Override
        public void enterG2(G2Context ctx) {
            addExpressionBinaryOperation(new ExpressionBinaryOperation(null, getBinaryOpForSymbol(ctx.getText()), null));
        }

        @Override
        public void enterG3(G3Context ctx) {
            addExpressionBinaryOperation(new ExpressionBinaryOperation(null, getBinaryOpForSymbol(ctx.getText()), null));
        }

        @Override
        public void enterG4(G4Context ctx) {
            addExpressionBinaryOperation(new ExpressionBinaryOperation(null, getBinaryOpForSymbol(ctx.getText()), null));
        }
        @Override
        public void enterPrefix_operator(Prefix_operatorContext ctx) {
            addPrefixUnaryOperation(new ExpressionUnaryOperation(getUnaryOpForSymbol(ctx.getText()), null));
        }

        @Override
        public void exitPrefix_operator(Prefix_operatorContext ctx) {
        }

        @Override
        public void enterLiteral_number(Literal_numberContext ctx) {
            addExpressionValue(new Literal(ctx.getText()));
        }

        @Override
        public void enterId_plus_indexers(Id_plus_indexersContext ctx) {
            String variableName = ctx.getText();
            Variable variable;

            // MATLAB prefixes enumeration constants with the enumeration name and a dot.  Strip that out.
            if (variableName.contains(".")) {
                variable = m_variableParser.get(variableName.split("\\.")[1]);
                // Assume the type matches.  Due to the fact we allow type aliasing, we can't check the types match.
            } else {
                variable = m_variableParser.get(variableName);
            }

            if (variable == null) {
                throw new IllegalArgumentException("No variable called " + variableName + " found!");
            }

            addExpressionValue(variable);
        }

        public Expression m_rootExpression;
        private int m_level = 0;
        private Map<String, Variable> m_variableParser;

        private static class ExpressionStackContainer {

            public int level;
            public ExpressionWithSubExpression op;

            public ExpressionStackContainer(int level, ExpressionWithSubExpression op) {
                this.level = level;
                this.op = op;
            }
        }
        Deque<ExpressionStackContainer> m_opStack = new ArrayDeque<ExpressionStackContainer>();

        private static class HiddenOpStackContainer {

            public int level, count;
            public BinaryOperation op;

            public HiddenOpStackContainer(int level, BinaryOperation op, int count) {
                this.level = level;
                this.op = op;
                this.count = count;
            }
        }
        Deque<HiddenOpStackContainer> m_hiddenOpStack = new ArrayDeque<HiddenOpStackContainer>();
    }
}
