/*
 * Copyright (C) 2014 Matthew Dawson <matthew@mjdsystems.ca>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in
 *       the documentation and/or other materials provided with the distribution
 *     * Neither the name of the McMaster Centre for Software Certification nor the names
 *       of its contributors may be used to endorse or promote products derived
 *       from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
 lexer grammar VariableParserLexer;

tokens { ID }

fragment IDRule: [a-zA-Z][a-zA-Z0-9_]*; //Variable identifier.  Start with letter, then alpha numerical allowed with underscores.

REGULAR_ID: IDRule -> type(ID), pushMode(VARIABLE_TYPE);

fragment WS : [ \t\r\n]+; // skip spaces, tabs, newlines
WS1: WS -> skip ;


mode PVS_EXPR_MODE;
PVS_EXPR: ~[\{\}]+ -> popMode;


mode VARIABLE_TYPE;
VAR_SEPARATOR: ',' -> popMode;
COLON: ':';
BOOL_TYPE: 'bool';
REAL_TYPE: 'real';
FIXEDT: 'fixedt' -> pushMode(FIXED_TYPE);
UNKNOWN_TYPE: IDRule;

OPEN_SUBTYPE: '{' -> pushMode(ID_FETCH);
PVS_EXPR_BEGIN: '|' -> pushMode(PVS_EXPR_MODE);
CLOSE_SUBTYPE: '}';

WS2: WS -> skip;

mode FIXED_TYPE;
NAT_SEPARATOR: ',';
OPEN_BRACKET: '(';
CLOSE_BRACKET: ')' -> popMode;
NAT: [0-9]+;

WS3: WS -> skip;


mode ID_FETCH;
SUBTYPE_ID: IDRule -> type(ID), popMode;

WS4: WS -> skip;
