/*
 * Copyright (C) 2014 Matthew Dawson <matthew@mjdsystems.ca>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in
 *       the documentation and/or other materials provided with the distribution
 *     * Neither the name of the McMaster Centre for Software Certification nor the names
 *       of its contributors may be used to endorse or promote products derived
 *       from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package ca.mcscert.jtet.tablularexpression;

import ca.mcscert.jtet.expression.Variable;
import ca.mcscert.jtet.expression.VariableCollection;

import java.util.ArrayList;
import java.util.List;

/**
 * @author Matthew Dawson <matthew@mjdsystems.ca>
 */
public class Table {
    public Table(String tableName, VariableCollection variables) {
        this.tableName = tableName;
        this.inputVariables = variables;
    }

    public void verifyTable() throws IllegalTableSetup {
        if (outputVariables.size() != variableOutputs.size()) {
            throw new IllegalTableSetup("The number of output variables and the number of outputs disagree!");
        }

        int leftCount = leftGrid.countTotalCases();
        int topCount = topGrid.countTotalCases();
        for (int i = 0; i < variableOutputs.size(); ++i) {
            TwoDimensionalGrid output = variableOutputs.get(i);
            if (output.sizeX() != topCount || output.sizeY() != leftCount) {
                throw new IllegalTableSetup("There are a different number of outputs then conditions for variable " + outputVariables.get(i) + "!");
            }
        }
    }

    public String getTableName() {
        return tableName;
    }

    public VariableCollection getInputVariables() {
        return inputVariables;
    }

    public HierarchicalGrid getLeftGrid() {
        return leftGrid;
    }

    public HierarchicalGrid getTopGrid() {
        return topGrid;
    }

    public List<TwoDimensionalGrid> getVariableOutputs() {
        return variableOutputs;
    }

    public List<Variable> getOutputVariables() {
        return outputVariables;
    }

    private String tableName;
    private VariableCollection inputVariables;
    private final HierarchicalGrid leftGrid = new HierarchicalGrid();
    private final HierarchicalGrid topGrid = new HierarchicalGrid();
    private final List<TwoDimensionalGrid> variableOutputs = new ArrayList<TwoDimensionalGrid>();
    private final List<Variable> outputVariables = new ArrayList<Variable>();

}
