/*
 * Copyright (C) 2013 Matthew Dawson <matthew@mjdsystems.ca>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in
 *       the documentation and/or other materials provided with the distribution
 *     * Neither the name of the McMaster Centre for Software Certification nor the names
 *       of its contributors may be used to endorse or promote products derived
 *       from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package ca.mcscert.jtet.expression;

import ca.mcscert.jtet.parsers.PVSSimpleParser;
import ca.mcscert.jtet.parsers.VariableParser;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.junit.runners.Parameterized;
import org.junit.runners.Parameterized.Parameter;
import org.junit.runners.Parameterized.Parameters;

import java.util.Arrays;
import java.util.Collection;

import static org.junit.Assert.*;

/**
 * @author Matthew Dawson <matthew@mjdsystems.ca>
 */
@RunWith(Parameterized.class)
public class EnumerationVariableTypeGenerationTest {
    @Parameters
    public static Collection<Object[]> data() {
        return Arrays.asList(new Object[][]{
                {new SMTLIBGenerator(),
                        "(= x e1)",
                        "(distinct x e1)",
                },
                {new CVC3Generator(),
                        "(x = e1)",
                        "(x /= e1)",
                }
        });
    }

    static private VariableType outputType = BooleanVariableType.Type;

    @Parameter(value = 0)
    public CheckerGenerator generator;

    @Parameter(value = 1)
    public String EqualExpectedOutput;

    @Parameter(value = 2)
    public String NotEqualExpectedOutput;

    @Before
    public void setUp() throws Exception {
        EnumerationVariableType enumType = new EnumerationVariableType("enum");
        enumType.enumerationValues().add("e1");

        variableParser.addCustomType("enum", enumType);
    }

    @Test
    public void EqualTest() {
        VariableCollection vars = variableParser.parseVariables("x:enum");
        Expression expr = PVSSimpleParser.parsePVSCode(vars, "x = e1");
        assertEquals(EqualExpectedOutput, expr.getCheckerOutput(generator, outputType));
    }

    @Test
    public void NotEqualTest() {
        VariableCollection vars = variableParser.parseVariables("x:enum");
        Expression expr = PVSSimpleParser.parsePVSCode(vars, "x /= e1");
        assertEquals(NotEqualExpectedOutput, expr.getCheckerOutput(generator, outputType));
    }

    @Test(expected=IllegalArgumentException.class)
    public void OtherOperatorsFailTest() {
        VariableCollection vars = variableParser.parseVariables("x:enum");
        Expression expr = PVSSimpleParser.parsePVSCode(vars, "x + e1");
        expr.getCheckerOutput(generator, vars.getVariables().get("x").type());
    }

    VariableParser variableParser = new VariableParser();
}
