/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package ca.mcscert.jtet.expression;

import static org.junit.Assert.*;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.junit.runners.JUnit4;

/**
 *
 * @author matthew
 */
@RunWith(JUnit4.class)
public class MatlabExpressionBinaryOperationTest {
    /**
     * Test of type method, of class ExpressionBinaryOperation.
     */
    @Test
    public void testWithSimpleRealVariablesAddedTogether() {
        Variable l = new Variable("a", new RealVariableType()), r = new Variable("b", new RealVariableType());
        ExpressionBinaryOperation instance = new ExpressionBinaryOperation(l, BinaryOperation.Addition, r);
        
        String CVC3Output = "(a + b)";
        VariableType expressionType = RealVariableType.Type;
        
        assertEquals(CVC3Output, instance.getCheckerOutput(generator, expressionType));
        assertEquals(expressionType, instance.actualOutputTypeForRequestedOutput(expressionType));
    }
    /**
     * Test of type method, of class ExpressionBinaryOperation.
     */
    @Test
    public void testWithSimpleFixedPointVariablesAddedTogether() {
        Variable l = new Variable("a", new FixedPointVariableType(true, 17, 4)), r = new Variable("b", new FixedPointVariableType(true, 17, 4));
        ExpressionBinaryOperation instance = new ExpressionBinaryOperation(l, BinaryOperation.Addition, r);
        
        String CVC3Output = "BVPLUS(17,a,b)";
        VariableType expressionType = new FixedPointVariableType(true, 17, 4);
        
        assertEquals(CVC3Output, instance.getCheckerOutput(generator, expressionType));
        assertEquals(expressionType, instance.actualOutputTypeForRequestedOutput(expressionType));
    }
    /**
     * Test of setLHS/RHS, verify they set variables correctly.
     */
    @Test
    public void testSettingsLHSAndRHS() {
        Variable l = new Variable("a", new RealVariableType()), r = new Variable("b", new RealVariableType());
        ExpressionBinaryOperation instance = new ExpressionBinaryOperation(l, BinaryOperation.Addition, r);
        
        //Reverse a+b to b+a.
        instance.setLHS(r);
        instance.setRHS(l);
        
        String CVC3Output = "(b + a)";
        VariableType expressionType = RealVariableType.Type;

        assertEquals(expressionType, instance.actualOutputTypeForRequestedOutput(expressionType));
        assertEquals(CVC3Output, instance.getCheckerOutput(new CVC3Generator(), expressionType));
    }
    
    /**
     * Test when RHS/LHS have different castable types.
     */
    @Test
    public void testCastableTypes() {
        Literal lit = new Literal("5");
        Variable var = new Variable("A", new RealVariableType());
        VariableType exprType;
        
        // Test when adding a Real var to a lit, type always becomes Real.
        exprType = RealVariableType.Type;
        assertEquals(exprType, new ExpressionBinaryOperation(lit, BinaryOperation.Addition, var).actualOutputTypeForRequestedOutput(exprType));
        assertEquals(exprType, new ExpressionBinaryOperation(var, BinaryOperation.Addition, lit).actualOutputTypeForRequestedOutput(exprType));

        // Test when comparing a Real var to a lit, type always becomes boolean.
        exprType = BooleanVariableType.Type;
        assertEquals(exprType, new ExpressionBinaryOperation(lit, BinaryOperation.Equals, var).actualOutputTypeForRequestedOutput(exprType));
        assertEquals(exprType, new ExpressionBinaryOperation(var, BinaryOperation.Equals, lit).actualOutputTypeForRequestedOutput(exprType));
    }

    /**
     * Test operations changing types (== doing REAL+REAL -> BOOLEAN)
     */
    @Test
    public void testCastingOperations() {
        Literal l = new Literal("5"), r = new Literal("6");
        ExpressionBinaryOperation exp = new ExpressionBinaryOperation(l, BinaryOperation.Equals, r);

        // Test when adding a Real var to a lit, type always becomes Real.
        VariableType expressionType = BooleanVariableType.Type;
        assertEquals(expressionType, exp.actualOutputTypeForRequestedOutput(expressionType));
        assertEquals("(5 = 6)", exp.getCheckerOutput(generator, expressionType));
    }

    /**
     * Test operations fails when a requested type is invalid!
     */
    @Test(expected=IllegalArgumentException.class)
    public void testInvalidOutputType() {
        Literal l = new Literal("5"), r = new Literal("6");
        ExpressionBinaryOperation exp = new ExpressionBinaryOperation(l, BinaryOperation.Equals, r);

        exp.getCheckerOutput(generator, RealVariableType.Type); //Returns a boolean, so will fail.
    }

    /**
     * Test doing multiple boolean operations AND'd together.
     */
    @Test
    public void testBooleanAndFromComparision() {
        Expression l = new Variable("a", new RealVariableType()), r = new Literal("6"),
                      l2 = new Variable("b", new RealVariableType()), r2 = new Literal("8");
        ExpressionBinaryOperation exp = new ExpressionBinaryOperation(new ExpressionBinaryOperation(l, BinaryOperation.Equals, r),
                BinaryOperation.BooleanAnd,
                new ExpressionBinaryOperation(l2, BinaryOperation.Equals, r2));

        // Test when adding a Real var to a lit, type always becomes Real.
        VariableType expressionType = BooleanVariableType.Type;
        assertEquals(expressionType, exp.actualOutputTypeForRequestedOutput(expressionType));
        assertEquals("((a = 6) AND (b = 8))", exp.getCheckerOutput(generator, expressionType));
    }

    CheckerGenerator generator = new CVC3Generator();
}
