/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package ca.mcscert.jtet.expression;

import static org.hamcrest.CoreMatchers.is;
import static org.junit.Assert.*;

import org.junit.Test;
import org.junit.runner.RunWith;
import org.junit.runners.JUnit4;

import java.util.HashMap;
import java.util.Map;

/**
 *
 * @author matthew
 */
@RunWith(JUnit4.class)
public class VariableTest {
    @Test
    public void exerciseVariable() {
        String name = "Var";
        VariableType type = new RealVariableType();
        
        Variable var = new Variable(name, type);
        
        assertSame(name, var.name());
        assertSame(type, var.type());
        assertEquals(name, var.getCheckerOutput(null, RealVariableType.Type));

        Map<String, Variable> varList = new HashMap<String, Variable>();
        varList.put(name, var);
        VariableCollection variableCollection = new VariableCollection(varList);

        CheckerGenerator generator = new CVC3Generator();
        assertEquals(name+":REAL;\n", generator.GenerateVariablesDeclaration(variableCollection));

        generator = new SMTLIBGenerator();
        assertEquals("(declare-fun " +name+ " () Real)\n", generator.GenerateVariablesDeclaration(variableCollection));
    }

    @Test
    public void testSubtypePredicate() {
        // Construct a basic variable.  Assume it is correct due to the above test.
        Variable var = new Variable("var", new RealVariableType());

        // Next create a subtype predicate.  For now, make it work over positive reals (excluding 0)
        Expression predicate = new ExpressionBinaryOperation(var, BinaryOperation.GreaterThen, new Literal("0"));
        var.type().setSubtypePredicate(predicate);

        // Verify the predicate took.
        assertThat(var.type().subtypePredicate(), is(predicate));

        // Make a variable list for the generators.
        Map<String, Variable> varList = new HashMap<String, Variable>();
        varList.put("var", var);
        VariableCollection variableCollection = new VariableCollection(varList);

        CheckerGenerator generator = new CVC3Generator();
        String varDecl = generator.GenerateVariablesDeclaration(variableCollection);

        assertThat(varDecl, is("var:REAL;\nASSERT (var > 0);\n"));

        generator = new SMTLIBGenerator();
        varDecl = generator.GenerateVariablesDeclaration(variableCollection);

        assertThat(varDecl, is("(declare-fun var () Real)\n(assert (> var 0.0))\n"));
    }

    @Test
    public void testOutputNameChangeInGenerator() {
        Variable var = new Variable("var", RealVariableType.Type);

        assertThat(var.name(), is("var"));
        assertThat(var.toString(), is("var"));
        assertThat(var.outputName(), is("var"));
        assertThat(var.getCheckerOutput(null, RealVariableType.Type), is("var"));

        var.setOutputName("new_name");

        assertThat(var.name(), is("var"));
        assertThat(var.toString(), is("var"));
        assertThat(var.outputName(), is("new_name"));
        assertThat(var.getCheckerOutput(null, RealVariableType.Type), is("new_name"));
    }

    // Verify handling of invalid type to getCheckerOutput
    @Test(expected=IllegalArgumentException.class)
    public void testInvalidOutputType() {
        String name = "Var";
        VariableType type = BooleanVariableType.Type;

        Variable var = new Variable(name, type);

        var.getCheckerOutput(null, RealVariableType.Type); //Returns a boolean, so will fail.
    }
}
