/*
 * Copyright (C) 2013 Matthew Dawson <matthew@mjdsystems.ca>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in
 *       the documentation and/or other materials provided with the distribution
 *     * Neither the name of the McMaster Centre for Software Certification nor the names
 *       of its contributors may be used to endorse or promote products derived
 *       from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package ca.mcscert.jtet.smtlibchecker;

import ca.mcscert.jtet.parsers.VariableParser;
import ca.mcscert.jtet.tablularexpression.HierarchicalCell;
import ca.mcscert.jtet.tablularexpression.HierarchicalGrid;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.junit.runners.JUnit4;

import java.util.List;

import static org.junit.Assert.*;

/**
 * @author Matthew Dawson <matthew@mjdsystems.ca>
 */
@RunWith(JUnit4.class)
public class CheckerRunnerTest {
    @Test
    public void TestRunningZ3() {
        HierarchicalGrid grid = new HierarchicalGrid();
        List<HierarchicalCell> topCells = grid.getSubHierarchy();

        topCells.add(new HierarchicalCell("x > 0"));
        topCells.add(new HierarchicalCell("x == 0"));
        topCells.add(new HierarchicalCell("0 > x"));

        List<HierarchicalCell> nextGrid = topCells.get(1).getSubHierarchy();
        nextGrid.add(new HierarchicalCell("z > 0"));
        nextGrid.add(new HierarchicalCell("z == 1"));
        nextGrid.add(new HierarchicalCell("0 > z"));

        //nextGrid.get(1).getSubHierarchy().add(new HierarchicalCell("z == 0"));

        List<CheckerRunnerResult> res = CheckerRunner.RunZ3(grid, variableParser.parseVariables("x,z"));
        assertEquals(2, res.size());
        assertEquals(2, res.get(0).SampleValues.size());
        assertEquals("0.0", res.get(0).SampleValues.get("x"));
        assertEquals("1.0", res.get(0).SampleValues.get("z"));

        assertEquals(2, res.get(1).SampleValues.size());
        assertEquals("0.0", res.get(1).SampleValues.get("x"));
        assertEquals("0.0", res.get(1).SampleValues.get("z"));
    }
    @Test
    public void TestRunningZ3WithNegative() {
        HierarchicalGrid grid = new HierarchicalGrid();
        List<HierarchicalCell> topCells = grid.getSubHierarchy();

        topCells.add(new HierarchicalCell("x > 0"));
        topCells.add(new HierarchicalCell("x == 0"));
        topCells.add(new HierarchicalCell("0 > x"));

        List<HierarchicalCell> nextGrid = topCells.get(1).getSubHierarchy();
        nextGrid.add(new HierarchicalCell("z > 0"));
        nextGrid.add(new HierarchicalCell("z == 0"));
        nextGrid.add(new HierarchicalCell("z == -1"));
        nextGrid.add(new HierarchicalCell("0 > z"));

        //nextGrid.get(1).getSubHierarchy().add(new HierarchicalCell("z == 0"));

        List<CheckerRunnerResult> res = CheckerRunner.RunZ3(grid, variableParser.parseVariables("x,z"));
        assertEquals(1, res.size());
        assertEquals(2, res.get(0).SampleValues.size());
        assertEquals("0.0", res.get(0).SampleValues.get("x"));
        assertEquals("(- 1.0)", res.get(0).SampleValues.get("z"));
    }
    @Test
    public void TestRunningZ3WithFractionalReturn() {
        HierarchicalGrid grid = new HierarchicalGrid();
        List<HierarchicalCell> topCells = grid.getSubHierarchy();

        topCells.add(new HierarchicalCell("x > 0"));
        topCells.add(new HierarchicalCell("x == 0"));
        topCells.add(new HierarchicalCell("x == 2.5"));
        topCells.add(new HierarchicalCell("0 > x"));

        List<CheckerRunnerResult> res = CheckerRunner.RunZ3(grid, variableParser.parseVariables("x"));
        assertEquals(1, res.size());
        assertEquals(1, res.get(0).SampleValues.size());
        assertEquals("(5.0 / 2.0)", res.get(0).SampleValues.get("x"));
    }
    @Test
    public void TestRunningZ3WithNegativeFractionalReturn() {
        HierarchicalGrid grid = new HierarchicalGrid();
        List<HierarchicalCell> topCells = grid.getSubHierarchy();

        topCells.add(new HierarchicalCell("x > 0"));
        topCells.add(new HierarchicalCell("x == 0"));
        topCells.add(new HierarchicalCell("x == -2.5"));
        topCells.add(new HierarchicalCell("0 > x"));

        List<CheckerRunnerResult> res = CheckerRunner.RunZ3(grid, variableParser.parseVariables("x"));
        assertEquals(1, res.size());
        assertEquals(1, res.get(0).SampleValues.size());
        assertEquals("(- (5.0 / 2.0))", res.get(0).SampleValues.get("x"));
    }

    VariableParser variableParser = new VariableParser();
}
