/*
 * Copyright (C) 2014 Matthew Dawson <matthew@mjdsystems.ca>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in
 *       the documentation and/or other materials provided with the distribution
 *     * Neither the name of the McMaster Centre for Software Certification nor the names
 *       of its contributors may be used to endorse or promote products derived
 *       from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package ca.mcscert.jtet.tablularexpression.test;

import ca.mcscert.jtet.expression.Variable;
import ca.mcscert.jtet.tablularexpression.HierarchicalCell;
import ca.mcscert.jtet.tablularexpression.IllegalTableSetup;
import ca.mcscert.jtet.tablularexpression.Table;
import ca.mcscert.jtet.tablularexpression.TwoDimensionalGrid;
import org.junit.Test;

public class TableTest {

    @Test(expected = IllegalTableSetup.class)
    public void testVerifyTableWithTooManyVariables() throws IllegalTableSetup {
        Table t = new Table(null, null);
        t.getOutputVariables().add(new Variable("a", null));

        t.verifyTable();
    }

    @Test(expected = IllegalTableSetup.class)
    public void testVerifyTableWithTooManyVariableOutputs() throws IllegalTableSetup {
        Table t = new Table(null, null);
        t.getVariableOutputs().add(new TwoDimensionalGrid());

        t.verifyTable();
    }

    @Test(expected = IllegalTableSetup.class)
    public void testVerifyTableWithWrongTopSize() throws IllegalTableSetup {
        Table t = new Table(null, null);
        t.getOutputVariables().add(new Variable("a", null));
        t.getVariableOutputs().add(new TwoDimensionalGrid());

        t.getLeftGrid().getSubHierarchy().add(new HierarchicalCell());
        t.getTopGrid().getSubHierarchy().add(new HierarchicalCell());
        t.getTopGrid().getSubHierarchy().add(new HierarchicalCell());

        t.verifyTable();
    }

    @Test(expected = IllegalTableSetup.class)
    public void testVerifyTableWithWrongLeftSize() throws IllegalTableSetup {
        Table t = new Table(null, null);
        t.getOutputVariables().add(new Variable("a", null));
        t.getVariableOutputs().add(new TwoDimensionalGrid());
        t.getVariableOutputs().get(0).resize(2, 5);

        t.getLeftGrid().getSubHierarchy().add(new HierarchicalCell());
        t.getLeftGrid().getSubHierarchy().get(0).getSubHierarchy().add(new HierarchicalCell());
        t.getLeftGrid().getSubHierarchy().get(0).getSubHierarchy().add(new HierarchicalCell());
        t.getLeftGrid().getSubHierarchy().add(new HierarchicalCell());
        t.getLeftGrid().getSubHierarchy().add(new HierarchicalCell());
        t.getTopGrid().getSubHierarchy().add(new HierarchicalCell());
        t.getTopGrid().getSubHierarchy().add(new HierarchicalCell());

        t.verifyTable();
    }
}
