/*
 * Copyright (C) 2013 Matthew Dawson <matthew@mjdsystems.ca>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in
 *       the documentation and/or other materials provided with the distribution
 *     * Neither the name of the McMaster Centre for Software Certification nor the names
 *       of its contributors may be used to endorse or promote products derived
 *       from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package ca.mcscert.jtet.tablularexpression.test;

import ca.mcscert.jtet.tablularexpression.Cell;
import ca.mcscert.jtet.tablularexpression.TwoDimensionalGrid;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.junit.runners.JUnit4;

import static org.hamcrest.CoreMatchers.*;
import static org.junit.Assert.*;

/**
 *
 * @author Matthew Dawson <matthew@mjdsystems.ca>
 */
@RunWith(JUnit4.class)
public class TwoDimensionalGridTest {
    @Test
    public void testEmptyConstructor() {
        TwoDimensionalGrid grid = new TwoDimensionalGrid();

        // By default, we should have a 1x1 grid.  Test that.
        assertThat(grid.sizeX(), is(1));
        assertThat(grid.sizeY(), is(1));
        Cell cell = grid.get(0,0);
        assertThat(cell, is(notNullValue()));
        assertThat(grid.get(0,0), is(cell));
    }

    @Test(expected = IllegalArgumentException.class)
    public void testNonPositiveXSize() {
        new TwoDimensionalGrid(-1, 2);
    }

    @Test(expected = IllegalArgumentException.class)
    public void testNonPositiveYSize() {
        new TwoDimensionalGrid(2, -1);
    }

    @Test(expected = IllegalArgumentException.class)
    public void testNonPositiveBothSides() {
        new TwoDimensionalGrid(-1, -2);
    }

    @Test(expected = IllegalArgumentException.class)
    public void testZeroSize() {
        new TwoDimensionalGrid(0, 2);
    }

    @Test(expected = IllegalArgumentException.class)
    public void testZeroYSize() {
        new TwoDimensionalGrid(2, 0);
    }

    @Test(expected = IllegalArgumentException.class)
    public void testZeroBothSides() {
        new TwoDimensionalGrid(0, 0);
    }
}
